/* Folder namespace */
impro.editor = {};

/* init the editor support */
impro.editor.init = function() {

	/* set a reference to the image editor */
	impro.editor.$editor = jQuery('#impro_editor');
	
	/* start with the editor closed */
	jQuery('#impro_editor_box').addClass('closed').find('.handlediv').hide();
		
	/* init events */
	impro.editor.initEvents();
	
	/* if the visual editor is enabled in the user preferences */
	if (jQuery('#edButtonPreview').size() > 0 || jQuery('#content-tmce').size() > 0) {
		/* hook up the switchEditors to find out when the editor is changed */
		/* TODO: This does not work for WordPress 3.2 */
		try {
			switchEditors.old_go = switchEditors.go;
			switchEditors.go = function(id, mode) {
				switchEditors.old_go(id, mode);
				impro.editor.onSwitchEditor(id, mode);
			};
		} catch(ex) {}
		
		/* tinyMCE will start initialization soon */
		var tinymce_timer = setInterval(function() {
			if (window.tinymce && window.tinymce.getInstanceById('content')) {
				clearInterval(tinymce_timer);
				
				/* prevent the user opening the image settings panel */
				jQuery('#impro_editor_box').find('.hndle').unbind('click');
				
				/* initialize the editor */
				impro.editor.onEditorInit(window.tinymce.getInstanceById('content'));
			}
		}, 300);
	} else {
		impro.err(imageproEditorL10N.enableTinyMce);
	}
};

/* get TinyMCE */
impro.editor.getTinyMCE = function() {
	return tinymce.getInstanceById('content');
};

/* fires when tinymce is initialized */
impro.editor.onEditorInit = function(editor) {
	/* now waiting for the iframe inside to render the post and have the body */
	var timer = setInterval(function() {
		var body = jQuery(window.tinymce.getInstanceById('content').contentAreaContainer).find('iframe').contents().find('body');
		if (body.size() > 0) {
			clearInterval(timer);
			
			/* link the click event to tiny mce */
			impro.editor.setTinyMCEInteractions(body, window.tinymce.getInstanceById('content'));
		}
	}, 300);
};

/* set the onclick event on the editor */
impro.editor.setTinyMCEInteractions = function(body, editor) {	
	
	/* register drop event to catch drops */
	setInterval(function() {
		/* because the ondrop event is not at all reliable
		 * not to say about it's cross browser support,
		 * I used a interval checking (polling) */
		var body = editor.dom.doc.body;
		var thumbs = jQuery(body).find('img[src*="\\/thumb\\/phpThumb\\.php"]');
		
		if (thumbs.size() > 0) {
			thumbs.each(function(i) {
				/* get source image */
				var thumb_w = jQuery(this).width();
				var thumb_h = jQuery(this).height();
				
				var src = jQuery(this).attr('src');
				var pic_src;
				
				/* get picture source */
				var match = /\/thumb\/phpThumb\.php\?src=([^&]*)&?/i.exec(src);
				if (match != null) {
					pic_src = match[1];
				}
				
				/* apply picture source and dimensions */
				if (pic_src) {
					jQuery(this).attr('src', pic_src).attr('data-mce-src', pic_src).attr('_mce_src', pic_src);
					jQuery(this).attr('width', thumb_w).attr('data-mce-width', thumb_w).attr('_mce_width', thumb_w);
					jQuery(this).attr('height', thumb_h).attr('data-mce-height', thumb_h).attr('_mce_height', thumb_h);
				}			
			});
		}
	}, 800);
		
	/* set the click event on body, then limit by images */
	body.bind('click', function(ev) {
		var target = jQuery(ev.target);
		/* only process image clicks */
		if (target.is('img')) {
			impro.editor.showEditor(target);
		} else {
			jQuery('#impro_editor_box').addClass('closed');
		}		
		return true;
	});
};

/* this is triggered when the picture is resized */
impro.editor.onMCEResize = function(ed, img) {
	
};

/* show the editor and populate the fields */
impro.editor.showEditor = function(target) {
	
	var current_src = jQuery(target).attr('src');
	var current_domain = document.location.protocol + '//' + document.location.host;
	
	/* if the image begins with http://my.domain/, it was uploaded with classic wordpress upload. ask for migrating to image pro */
	if (current_src.indexOf(current_domain + '/') == 0) {
		if (confirm(imageproEditorL10N.convertToImagePro)) {
			var new_src = current_src.replace(current_domain, '');
			jQuery(target).attr('src', new_src).attr('data-mce-src', new_src).attr('_mce_src', new_src);
			impro.editor.showEditor(target);
		}
		return;
	}
	
	/* if does not begin with /, it means it's from an external domain */
	if (current_src[0] != "/") {
		impro.err(imageproEditorL10N.differentDomain);
		return;
	}
	
	/* open the editor panel */
	jQuery('#impro_editor_box').removeClass('closed');
	
	/* remember the clicked image (currently in editor) */
	impro.editor.$clickedImage = target;
	
	/* parse clicked image's path */
	var path = impro.editor.parseUrl(target.attr('src'));
	
	/* remember the old source */
	impro.editor.$clickedImage.get(0).old_src = target.attr('src');
	
	/* show thumb image */
	path['w'] = 50;
	path['h'] = 50;
	path['zc'] = 0;	
	var url = impro.editor.createThumbUrl(path);
	impro.editor.$editor.find('.thumb').attr('src', url);
	
	/* set alt and title */
	jQuery('.impro_alt').val(target.attr('alt'));
	jQuery('.impro_title').val(target.attr('title'));
	
	/* set align mode */
	var align = "alignnone";
	if (target.hasClass('alignleft')) {
		align = "alignleft";
	} else if (target.hasClass('aligncenter')) {
		align = "aligncenter";
	} else if (target.hasClass('alignright')) {
		align = "alignright";
	}
	jQuery('.impro_align').val(align);
};

/* change a given img element to what is contained in props */ 
impro.editor.resize = function(el, props) {
	if (!jQuery(el).is('img')) {	/* el must be an img tag */
		return null;
	}
	
	if (!(el instanceof jQuery)) {	/* make the element jQuery if not already */
		el = jQuery(el);
	}
	
	var el_dom = el.get(0);	/* also keep a DOM reference */
	
	var current_props = impro.editor.parseUrl(el.attr('src'));	/* get the current properties */
	
	/* if we could not extract properties from the source, get them from the image tag properties */
	if (!current_props['w']) {
		current_props['w'] = el_dom.width || el.width();
	}
	if (!current_props['h']) {
		current_props['h'] = el_dom.height || el.height();
	}
	
	/* copy properties */
	if (props['src']) {
		current_props['src'] = props['src'];
	}
	if (props['w']) {
		current_props['w'] = props['w'];
	}
	if (props['h']) {
		current_props['h'] = props['h'];
	}
	if (props['zc']) {
		current_props['zc'] = props['zc'];
	}
	
	var url = impro.editor.createThumbUrl(current_props);
	
	/* set preview */
	el.attr('src', props['src']);
	el.attr('width', props['w']);
	el.attr('height', props['h']);
	
	/* save */
	el.attr('_mce_src', props['src']);
	el.attr('_mce_width', props['w']);
	el.attr('_mce_height', props['h']);
	el.attr('data-mce-src', props['src']);
	el.attr('data-mce-width', props['w']);
	el.attr('data-mce-height', props['h']);
};

impro.editor.initEvents = function() {
	var changeProps = function() {
		/* preview */
		impro.editor.$clickedImage.attr('alt', jQuery('.impro_alt').val());
		impro.editor.$clickedImage.attr('title', jQuery('.impro_title').val());
		impro.editor.$clickedImage.removeClass('alignnone').removeClass('alignleft').removeClass('aligncenter').removeClass('alignright').addClass(jQuery('.impro_align').val());
		
		/* save */
		impro.editor.$clickedImage.attr('_mce_alt', jQuery('.impro_alt').val());
		impro.editor.$clickedImage.attr('_mce_title', jQuery('.impro_title').val());
		impro.editor.$clickedImage.attr('_mce_class', impro.editor.$clickedImage.attr('class'));
	};
	
	jQuery('.impro_align, .impro_alt, .impro_title').change(changeProps).keyup(changeProps);
	
	jQuery('.impro_choose').click(function() {
		jQuery('#impro_folder_box #vimg-list').addClass('impro_choosebg');
		jQuery.scrollTo('#impro_folder_box',800);
		impro.editor.$chosen = impro.editor.$clickedImage;
	});
};

impro.editor.createThumbUrl = function(params) {
	var path = impro.url + '/src/thumb/phpThumb.php?';
	var tmp = [];
	for (var p in params) {
		if (['w','h','src','zc','q'].indexOf(p) === -1) {
			continue;
		}
		if (params.hasOwnProperty(p)) {
			tmp.push(p+'='+params[p]);
		}
	}
	return path + tmp.join('&');
}

/* fires when the editor is changed */
impro.editor.onSwitchEditor = function(id, mode) {
	
};

impro.editor.parseUrl = function(url) {
	var path = {};
	
	/* if begins with http(s?):// protocol */
	var is_http = /^https?:\/\/$/i.test(url);
	/* if contains phpThumb.php */
	var is_thumb = url.indexOf('/src/thumb/phpThumb.php') !== -1;
	
	if (is_thumb) {
		path['thumb'] = true;
		
		url.split('?')[1].split('&').forEach(function(v){
			var tmp = v.split('=');
			
			/* we have the width, height, quality and zoom-crop */
			if (tmp[0] === 'w' || tmp[0] === 'h' || tmp[0] === 'q' || tmp[0] === 'zc') {
				tmp[1] = parseInt(tmp[1], 10);
				/* not a numeric value, don't consider */
				if (isNaN(tmp[1])) {
					return;
				}
			}
			
			path[tmp[0]] = tmp[1];
		});
		
		path['url'] = url;
	} else {
		path['thumb'] = false;
		path['url'] = url;
		path['src'] = path['url'];
	}
	
	return path;
};

jQuery(function() {
	impro.editor.init();
});